
#### Function: multiple_gbm ####
multiple_gbm <- function(formula, data_to_train, n_outputs,best_hyper,train.fraction) {
  # randomize data
  random_index <- sample(1:nrow(data_to_train), nrow(data_to_train))
  random_data_to_train <- data_to_train[random_index, ]
  list_gbm <- list()
  for (index in 1:n_outputs) {
    col_names_output <- paste0("PC",index)
    Vx_formula <- paste0( col_names_output,formula)
    gbm1 <- gbm(as.formula(Vx_formula),
                data = random_data_to_train,
                distribution = "gaussian",
                
                n.trees               = best_hyper$n.trees[index],          
                interaction.depth     = best_hyper$interaction.depth[index], 
                shrinkage             = best_hyper$shrinkage[index],  
                n.minobsinnode        = best_hyper$n.minobsinnode[index],    
                bag.fraction          = best_hyper$bag.fraction[index],    
                cv.folds              = best_hyper$cv.folds[index],                 
                keep.data             = best_hyper$keep.data[index], 
                train.fraction = train.fraction,  
                verbose = FALSE,
                n.cores = 1)
    
    list_gbm[[index]] <- gbm1
  }
  return(list_gbm)
}


#### Function: optimezed_multiple_gbm ####
# Goal : build surrogate model based on gradient boosting machine method for several outputs
# Created on: 15/12/2021
optimized_multiple_gbm <- function(formula,
                                   data_to_train,
                                   n_outputs,
                                   n.trees,
                                   hyper_grid,
                                   train.fraction,
                                   n.cores, # will use all cores by default
                                   verbose) {
  # randomize data
  random_index <- sample(1:nrow(data_to_train), nrow(data_to_train))
  random_data_to_train <- data_to_train[random_index, ] 
  list_gbm <- list()
  for (index in 1:n_outputs) {
    col_names_output <- paste0("PC",index)
    Vx_formula <- paste0( col_names_output,formula)
    gbm1 <- gbm(as.formula(Vx_formula),
                data = random_data_to_train,
                distribution = "gaussian",
                n.trees = n.trees,
                interaction.depth = hyper_grid$interaction.depth,
                shrinkage = hyper_grid$shrinkage,
                n.minobsinnode = hyper_grid$n.minobsinnode,
                bag.fraction = hyper_grid$bag.fraction,
                cv.folds = hyper_grid$cv.folds,                
                train.fraction = train.fraction,  
                keep.data = hyper_grid$keep.data, 
                verbose = verbose,
                n.cores = n.cores)  
    list_gbm[[index]] <- gbm1
  }
  return(list_gbm)
}


#### Function: multiple_predict ####
# Goal : predict an array of outputs from a list of surrogate models
# Created on: 9/11/2021
multiple_predict <- function(list_predict, newdata, n_outputs) {
  # list_predict : list of surrogate models used to create ouput array
  # newdata : array or data.frame containing model parameter values to evaluate the surrogate
  # n_outputs :  dimension of the output array 
  res <- NULL
  for (index in 1:n_outputs) {
    single_res <- predict( list_predict[[index]] , newdata=newdata )
    res <- cbind(res,single_res)
  }
  return(res)
}

#### Function: surrogate_model ####
# Goal : Evaluate multidimensional surrogate model output 
# Created on: 9/11/2021
surrogate_model <- function(newdata, c_gbm, data_to_train.pca, n_outputs,i=NULL,log_data=NULL) {
  
  # newdata: array or data.frame containing model parameter values to evaluate the surrogate
  # c_gbm: multidimensional surogate model created by multiple_gbm (or similar functions: to be implmented)
  # data_to_train.pca: PCA cretaed using the training data
  # n_outputs :  dimension of the reduced output array
  # i: if provided provides only results for specific output dimension
  
  res <- multiple_predict( c_gbm, newdata=newdata , n_outputs)
  res <- res[,] %*% t(data_to_train.pca$rotation[,1:n_outputs])
  if(all(data_to_train.pca$scale != FALSE)){
    res  <- scale(res, center = FALSE , scale=1/data_to_train.pca$scale)
  }
  if(all(data_to_train.pca$center != FALSE)){
    res <- scale(res, center = -1 * data_to_train.pca$center, scale=FALSE)
  }
  
  
  if(!is.null(i)) {res <- res[,i]}
  #res <- newdata[,1]
  if(log_data) res <- exp(res)
  
  return(res)
  
}

#### Function: mulitple_sobol ####
surrogate_model_forSobol_onPC <- function(newdata,c_gbm) 
{
  res <- NULL
  n_outputs <- length(c_gbm)
  colnames(newdata) <- param_names
  for (index in 1:n_outputs) {
    single_res <- predict( c_gbm[[index]] , newdata=newdata )
    res <- cbind(res,single_res)
  }
  dimnames(res)[[2]][ncol(res)] <- ""
  return(res)
}


surrogate_model_forSobol_onPhysicalSpace <- function(newdata,c_gbm,data_to_train.pca,log_data=NULL) 
{

  n_outputs <- length(c_gbm)
  colnames(newdata) <- param_names
  res <- surrogate_model(newdata, c_gbm, data_to_train.pca, n_outputs,i=NULL,log_data=TRUE) 
  dimnames(res)[[2]][ncol(res)] <- ""
  return(res)
}


#### Function: plot_field ####
# Goal : Reshape data to 2D matrix for 1D Pisces fields OJO dimensions are hard-coded!!!
# Created on: 16/11/2021
plot_field <- function(var,i_row,data_for_plot,n_z = 47,n_t = 73,deptht,mldkz)
{
  my.padding <- list(layout.heights = list(
    top.padding = 0,
    main.key.padding = 0,
    key.axis.padding = 0,
    axis.xlab.padding = 0,
    xlab.key.padding = 0,
    key.sub.padding = 0),
    layout.widths = list(
      left.padding = 0,
      key.ylab.padding = 0,
      ylab.axis.padding = 0,
      axis.key.padding = 0,
      right.padding = 0)
  )
  
  i_start <- n_z*n_t*(var-1) + 1
  i_end   <- n_z*n_t*(var)
  par(mar=c(0,0,0,0))
  X = seq(1,365,5)
  Y = deptht[1:n_z]
  data_to_plot <- expand.grid(X=Y,Y=X)
  data_to_plot$Z <- data_for_plot[i_row,i_start:i_end]
  y_mld <- mldkz
  
  # Generate the levelplot and add a curve
  levelplot( Z~Y*X, data_to_plot,
             col.regions=viridis(31),
             xlab =NULL, ylab = "depth (m)", ylim = c(max(Y),min(Y)), 
             scales=list( y=list(at=c(200, 800), labels=c(200, 800)), x=list(at=c(32, 152, 274), labels=c("Feb","Jun","Oct")) ),
             #colorkey=list( at=seq(0,1, length=30)),
             par.settings = my.padding, panel = function(x, y, z, ...) {
               panel.levelplot(x, y, z , ...)
               panel.xyplot(X, y_mld, type = "l", col = "white",lwd=2)
             })  
  
}

plot_allfields <- function(S1_param){
  S1_X <-readRDS(paste0(S1_param,".rds"))
  print(paste(S1_param,"Mean:", mean(S1_X), "Max:",max(S1_X) ))
  dim(S1_X) <- c(1,length(S1_X))
  
  p1 <-  plot_field(1,1,S1_X )
  p2 <-  plot_field(2,1,S1_X )
  # p3 <-  plot_field(3,1,S1_X )
  # p4 <-  plot_field(4,1,S1_X )
  # p5 <-  plot_field(5,1,S1_X )
  # p6 <-  plot_field(6,1,S1_X )
  # p7 <-  plot_field(7,1,S1_X )
  # p8 <-  plot_field(8,1,S1_X )
  pdf(file = paste0(S1_param,".pdf"), width = 30, height = 2)
  grid.arrange(p1,p2,ncol=3)
  # grid.arrange(p1,p2,p3,p4,p5,p6,p7,p8, ncol=11)
  dev.off()
}
