#### LOAD config_file
args = commandArgs(TRUE)
if (length(args) > 0) {
  configFile_path <- args[1]
} else {
  cat("\n***WARNING: no ConfigFile was provided. Using default ConfigFile.\n")
}
config       <- yaml::yaml.load_file(configFile_path)
file         <- config$file
nparams      <- config$nparams
sim_data     <- config$sim_data
image_folder <- config$image_folder
n_outputs    <- config$n_outputs
repo         <- config$repo

#### Load external functions
source(paste0(repo,"UQtools4es.R"))
library(gbm)

#### Load data
reduced_train  <- readRDS(paste0(sim_data,"reduced_train.RDS"))
mypca          <- readRDS(paste0(sim_data,"mypca.RDS"))
data_to_validate <- readRDS(paste0(sim_data,"data_to_validate.RDS"))
# data_to_validate <- data_to_validate[1:100,]
n_dim <- ncol(data_to_validate) - nparams # ouput dimenions
#### BUILD surrogate

# create hyperparameter grid
hyper_grid <- expand.grid(
  shrinkage         = config$shrinkage,
  interaction.depth = config$interaction.depth,
  n.minobsinnode    = config$n.minobsinnode,
  bag.fraction      = config$bag.fraction,
  cv.folds          = config$cv.folds,
  keep.data         = config$keep.data,
  optimal_trees     = 0, # a place to dump results
  min_RMSE          = 0  # a place to dump results
)

# assumed hyperparameters
train.fraction  <- config$train.fraction
n.trees	 	<- config$n.trees 
n.cores		<- config$n.cores	

# optimezed model definition
model_params_formula <- "~"
for (param in colnames(reduced_train[1,(n_outputs+1):ncol(reduced_train)]) ) {
  model_params_formula <- paste(model_params_formula, "+",param)
}
 
# grid search 
optimal <- list()
for (pc_i in (1:n_outputs)) { optimal[[pc_i]] <- hyper_grid}
for(i in (1:nrow(hyper_grid)) ) {  
  # train model
  c_gbm   <- optimized_multiple_gbm(model_params_formula,
                                    reduced_train,
                                    n_outputs,
                                    n.trees = n.trees,
                                    hyper_grid[i,],
                                    train.fraction = train.fraction,
                                    n.cores = n.cores, # will use all cores by default
                                    verbose = FALSE)
  # add min training error and trees to grid
  for (pc_i in (1:n_outputs)) {
    optimal[[pc_i]]$optimal_trees[i] <- which.min(c_gbm[[pc_i]]$valid.error)
    optimal[[pc_i]]$min_RMSE[i]      <- sqrt( min(c_gbm[[pc_i]]$valid.error) )
  }
  print(paste0("nrow_hyperparams =",i))
  print("Params values :")
  print(hyper_grid[i,])
  print("min_RMSE :")
  print(optimal[[1]]$min_RMSE[i]  )
}
# optimal set of hyperparameters
best_hyper <- data.frame(n.trees=numeric(n_outputs),
                         interaction.depth=numeric(n_outputs),
                         shrinkage=numeric(n_outputs),
                         n.minobsinnode=numeric(n_outputs),
                         bag.fraction=numeric(n_outputs),
                         cv.folds=numeric(n_outputs),
                         keep.data=logical(n_outputs))
for (pc_i in (1:n_outputs)) {
  index_best                         <-  which.min(  optimal[[pc_i]]$min_RMSE )
  best_hyper$n.trees[pc_i]           <-  optimal[[pc_i]]$optimal_trees[index_best]
  best_hyper$interaction.depth[pc_i] <-  optimal[[pc_i]]$interaction.depth[index_best]
  best_hyper$shrinkage[pc_i]         <-  optimal[[pc_i]]$shrinkage[index_best]
  best_hyper$n.minobsinnode[pc_i]    <-  optimal[[pc_i]]$n.minobsinnode[index_best]
  best_hyper$bag.fraction[pc_i]      <-  optimal[[pc_i]]$bag.fraction[index_best]
  best_hyper$cv.folds[pc_i]          <-  optimal[[pc_i]]$cv.folds[index_best]         
  best_hyper$keep.data[pc_i]         <-  optimal[[pc_i]]$keep.data[index_best]
}

# Set the optimal gbm for each PC
c_gbm   <- multiple_gbm(model_params_formula,
                                  reduced_train,
                                  n_outputs,
                                  best_hyper,
                                  train.fraction = train.fraction)

# Compute sensitivity based on relative influencei at each PC. At each split in each tree, gbm computes the improvement in MSE. gbm then averages the improvement made by each variable across all the trees that the variable is used. The variables with the largest average decrease in MSE are considered most important.
# rel.inf <- summary(c_gbm[[1]], 
#                    cBars = nparams,
#                    method = relative.influence, # also can use permutation.test.gbm
#                    las = 2)[1]
# global_rel.inf <- array(data=0,dim=n_outputs)
# for (pc_i in (1:n_outputs)) {
#   single_rel.inf <- summary(c_gbm[[pc_i]], 
#                             cBars = nparams,
#                             method = relative.influence, # also can use permutation.test.gbm
#                             las = 2
#   )
#   colnames(single_rel.inf)[2] <- colnames(reduced_train)[pc_i]
#   rel.inf <- cbind(rel.inf,single_rel.inf[2]) # relative influence needs to be reordered before combined!!
#   global_rel.inf <- global_rel.inf + single_rel.inf[2] * summary(mypca)$importance[2,pc_i]
# }
# print(global_rel.inf)

##### Compute relative error of the surrogate based on the training data
res <- surrogate_model(data_to_validate[,(n_dim+1):ncol(data_to_validate)],c_gbm, mypca, n_outputs,log_data=TRUE) # Evaluate surrogate using model parameters from data_to_validate
original <- data_to_validate[,1:n_dim]  
# Compute square error from PCA + Surrogate using validation data
norm_diff = norm( as.matrix(data_to_validate[,1:n_dim] - res[,]) , type = "f")  
norm_data = norm( as.matrix(data_to_validate[,1:n_dim])          , type = "f")  
rel_err = 100 * norm_diff / norm_data 
print(paste(rel_err,"%"))



# library(viridis)
# var <- 1
# i_row <- 2
# data_for_plot = res
# plot_field(var, i_row, data_for_plot)
# 
# library(viridis)
# var <- 1
# i_row <- 2
# data_for_plot = original
# plot_field(var, i_row, data_for_plot)


#### save data
saveRDS(c_gbm,  file = paste0(sim_data,"c_gbm.RDS"))
